import time
import imp
import asyncio

from sanic.views import HTTPMethodView
from sanic.response import text
from sanic.response import json
from sanic.exceptions import ServerError

class BaseView(HTTPMethodView):
  def __init__(self, method, module, path, timeout, file_upload = False):
      self.method = method
      self.file_upload = file_upload
      try: 
          file, pathname, desc = imp.find_module(module,[path])
          self.moduleobj = imp.load_module(module, file, pathname, desc)
      except Exception as e:
          raise

  def request_to_message(self, request):
      message = dict()
      try:
          message["url_content"] = request.args
          message["form_content"] = request.form
          message["json_content"] = request.json
      except Exception:
          pass
      return message

  def return_check(self, response):
      if isinstance(response, dict):
          return json(response)
      else:
          raise ServerError("The `dict` is expected. Please check the type of the callback", status_code=401)

  async def get(self, request):
      if 'get' not in self.method:
          return ServerError("Not support", status_code=400)
      message = self.request_to_message(request)
      handler = "self.moduleobj.get.delay"
      callback = eval(handler)(message)
      while(not callback.ready()):
          await asyncio.sleep(1)      
      response = callback.result
      return self.return_check(response)

  async def post(self, request):
      if 'post' not in self.method:
          return ServerError("Not support", status_code=400)
      message = self.request_to_message(request)
      if self.file_upload:
           message = request
      handler = "self.moduleobj.post.delay"
      callback = eval(handler)(message)
      while(not callback.ready()):
          await asyncio.sleep(1)      
      response = callback.result
      return self.return_check(response)

  async def put(self, request):
      if 'put' not in self.method:
          return ServerError("Not support", status_code=400)
      message = self.request_to_message(request)
      handler = "self.moduleobj.put.delay"
      callback = eval(handler)(message)
      while(not callback.ready()):
          await asyncio.sleep(1)      
      response = callback.result
      return self.return_check(response)

  async def patch(self, request):
      if 'patch' not in self.method:
          return ServerError("Not support", status_code=400)
      message = self.request_to_message(request)
      handler = "self.moduleobj.patch.delay"
      callback = eval(handler)(message)
      while(not callback.ready()):
          await asyncio.sleep(1)      
      response = callback.result
      return self.return_check(response)
  
  async def delete(self, request):
      if 'delete' not in self.method:
          return ServerError("Not support", status_code=400)
      message = self.request_to_message(request)
      handler = "self.moduleobj.delete.delay"
      callback = eval(handler)(message)
      while(not callback.ready()):
          await asyncio.sleep(1)      
      response = callback.result
      return self.return_check(response)



import os
import configparser
import logging
import signal
import subprocess

from asyncio import get_event_loop
from sanic import Sanic
from sanic.response import json
from sanic.config import Config
from sanic.response import text


from .controller import BaseView

class Allegro(object):
    def __init__(self, name):
        self.log = logging.getLogger('allegro')

        formatter = logging.Formatter(
                    "%(asctime)s: %(levelname)s: %(message)s")
        handler = logging.StreamHandler()
        handler.setFormatter(formatter)
        self.log.addHandler(handler)
        self.log.setLevel(logging.INFO)

        self.app = Sanic(name)
    
    def initialize(self, config_path):
         
        try:
            self.cf = configparser.ConfigParser()
            self.cf.read(config_path)
         
            self.host = self.cf["basic"]["bind_host"]
            self.port = int(self.cf["basic"]["bind_port"])
            self.root_path = self.cf["basic"]["root_path"]
            self.api_worker = int(self.cf["basic"]["api_worker"])
            self.pid_path = self.cf["basic"]["pid_path"]
            self.timeout = int(self.cf["basic"]["timeout"])
            self.app.config.REQUEST_TIMEOUT=self.timeout

        except Exception as e:
            self.log.exception("Error ocurred when trying to the config file Info: %s" % str(e))
            raise

    def init_route(self):
        services = self.cf["service"]["keys"].replace(' ', '').split(',')
        for service in services:
            uri = self.cf[service]["uri"]
            module = self.cf[service]["module"]
            file_upload = self.cf[service]["file_upload_enable"]
            method = self.cf[service]["method"].lower().replace(' ','').split(',')
            self.app.add_route(BaseView.as_view(method, module, self.root_path, self.timeout, file_upload), uri)

    def start(self):
        try:
            self.init_route()
            with open(self.pid_path, "a") as f:
                f.write(str(os.getpid())+"\n")

            services = self.cf["service"]["keys"].split(',')
            os.system('cd %s' % self.root_path)
            for service in services:
                module = self.cf[service]["module"]
                eventlet_enabled = eval(self.cf[service]['eventlet_enabled'])
                if eventlet_enabled:
                    eventlet_pool = int(self.cf[service]["eventlet_pool"])
                    subprocess.call('celery worker -A %s --concurrency 1   -l info -P eventlet -c %s -n %s &' % (module, eventlet_pool, module), shell=True)
                else:
                    workers = int(self.cf[service]["workers"])
                    for i in range(workers):
                        subprocess.call('celery worker -A %s --concurrency %s   -l info -n %s%s &' % (module, workers, module, i), shell=True)

            self.log.info("Starting Consumer service...")
            self.app.add_task(self.save_pid())
            self.app.run(host=self.host, port=self.port, workers=self.api_worker)
        except Exception as e:
            self.log.exception("Error ocurred when trying to the config file Info: %s" % str(e))
            raise

    async def save_pid(self):
        with open(self.pid_path, "a") as f:
            f.write(str(os.getpid())+"\n")

    def stop(self):
        try:
            self.log.info("Staring to termina the processes...")
            with open(self.pid_path, "r") as f:
                processes = f.readlines()
                for p in processes:
                    try:
                        a = os.kill(int(p[:-1]), signal.SIGKILL)
                    except Exception as e:
                        self.log.error(e)
            open(self.pid_path, "w+")
            os.system("pkill -9 -f 'celery worker'")
            self.log.info("All the processes are terminated.")
        except Exception as e:
            log.exception("Error ocurred when trying to kill the processes Info: %s" % str(e)) 
            raise

